#!/usr/bin/env ruby

# See also: http://stackoverflow.com/questions/43375157/whats-the-best-way-to-write-a-has-many-through-so-that-it-just-uses-a-plain

begin
  require 'bundler/inline'
rescue LoadError => e
  $stderr.puts 'Bundler version 1.10 or later is required. Please update your Bundler'
  raise e
end

gemfile(false) do
  source 'https://rubygems.org'
  gem 'activerecord', '5.0.2'
  gem 'sqlite3'
end

require 'active_record'
require 'minitest/autorun'
require 'logger'

Minitest::Test = MiniTest::Unit::TestCase unless defined?(Minitest::Test)

ActiveRecord::Base.establish_connection(adapter: 'sqlite3', database: ':memory:')
ActiveRecord::Base.logger = Logger.new(STDOUT)

ActiveRecord::Schema.define do
  create_table :companies, force: true do |t|
  end

  create_table :records, force: true do |t|
    t.integer :company_id, index: true
  end

  create_table :users, force: true do |t|
    t.integer :company_id, index: true
  end
end

class Company < ActiveRecord::Base
  has_many :records
end

class Record < ActiveRecord::Base
  belongs_to :company
end

class User < ActiveRecord::Base
  belongs_to :company
  has_many :records, through: :company

  # # Produces the correct query:
  # has_many :records, foreign_key: :company_id, primary_key: :company_id

  # # Alternatively:
  # def records
  #   Record.where(company_id: company_id)
  # end
end


class QueryTest < Minitest::Test
  def test_sql
    company = Company.create()
    record = Record.create(company: company)
    user = User.create(company: company)

    # Currently generates SQL with an unnecessary INNER JOIN:
    #
    # SELECT "records".* FROM "records" INNER JOIN "companies" \
    # ON "records"."company_id" = "companies"."id" WHERE "companies"."id" = 1

    assert_equal user.records.to_sql,
                 'SELECT "records".* FROM "records" WHERE "records"."company_id" = 1'
  end
end
